require 'json'
require "redis/distributed_logger/version"

class Redis
  class DistributedLogger
    # Low-level information, mostly for developers.
    DEBUG = 0
    # Generic (useful) information about system operation.
    INFO = 1
    # A warning.
    WARN = 2
    # A handleable error condition.
    ERROR = 3
    # An unhandleable error that results in a program crash.
    FATAL = 4
    # An unknown message that should always be logged.
    UNKNOWN = 5

    # for compatibily with classic Logger methods
    attr_accessor :datetime_format, :level

    def initialize( redis, key, stdout = nil )
      @_redis = redis
      @_key = key
      @_stdout = stdout
      @_level = INFO
    end

    def <<(msg)
      @_redis.lpush( @_key, { at: Time.now.to_i, severity: nil, progname: nil, message: msg }.to_json )
    end

    def add( severity, message = nil, progname = nil, &block )
      @_redis.lpush( @_key, { at: Time.now.to_i, severity: severity, progname: block_given? ? message : progname, message: block_given? ? yield : message }.to_json )
    end
    alias_method :log, :add

    def close
      # nothing
    end

    def debug?
      @_level <= DEBUG
    end

    def error?
      @_level <= ERROR
    end

    def fatal?
      @_level <= FATAL
    end

    def info?
      @_level <= INFO
    end

    def warn?
      @_level <= WARN
    end

    def debug( arg = nil, &block )
      add( DEBUG, block_given? ? yield : arg, block_given? ? arg : nil )
    end

    def error( arg = nil, &block )
      add( ERROR, block_given? ? yield : arg, block_given? ? arg : nil )
    end

    def fatal( arg = nil, &block )
      add( FATAL, block_given? ? yield : arg, block_given? ? arg : nil )
    end

    def info( arg = nil, &block )
      add( INFO, block_given? ? yield : arg, block_given? ? arg : nil )
    end

    def unknown( arg = nil, &block )
      add( UNKNOWN, block_given? ? yield : arg, block_given? ? arg : nil )
    end

    def warn( arg = nil, &block )
      add( WARN, block_given? ? yield : arg, block_given? ? arg : nil )
    end
  end
end
